#!/usr/bin/env python

'''

    @version $Id: __init__.py 7231 2007-05-20 22:53:21Z davidfancella $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005, 2007 by David Fancella

'''

from PyQt4 import QtGui, QtCore

from libacme import paths
from libacme import resource_parser

from libacmegui import toolView
from libacmegui.dialogs import GetSaveFile

from libacmegui.editors import mapEditor
from libacmegui.editors import genericEditor

class EditorStack(QtGui.QStackedWidget):
    viewer = None
    sourcer = None
    viewmode = None
    
    def __init__(self, parent, viewmode="viewer"):
        QtGui.QStackedWidget.__init__(self, parent)
        self.viewmode = viewmode
        
    def LoadResource(self, theResource):
        if self.sourcer is not None:
            self.sourcer.LoadResource(theResource)
        if self.viewer is not None:
            self.viewer.LoadResource(theResource)
        
    def HasChanges(self):
        return self.currentWidget().HasChanges()
        
    def SetViewMode(self, viewmode):
        self.viewmode = viewmode
        
    def idstr(self):
        if self.viewer is not None:
            return self.viewer.idstr()
        else:
            return self.sourcer.idstr()
        
    def AddViewWidget(self, viewer):
        self.viewer = viewer
        self.addWidget(viewer)
        
    def AddSourceWidget(self, source):
        self.sourcer = source
        self.addWidget(source)
        
    def FocusViewer(self, viewMode):
        if self.viewer is not None and self.viewmode == "viewer":
            if viewMode == "source":
                self.viewer.CommitToDom()
                self.sourcer.UpdateFromDom()
                self.setCurrentWidget(self.sourcer)
                self.viewmode = viewMode
            else:
                self.setCurrentWidget(self.viewer)
        elif self.sourcer is not None:
            if viewMode == "viewer" and self.viewer is not None:
                self.sourcer.CommitToDom()
                self.viewer.UpdateFromDom()
                self.setCurrentWidget(self.viewer)
                self.viewmode = viewMode
            else:
                self.setCurrentWidget(self.sourcer)

class mainView(QtGui.QWidget):
    editorList = None
    
    def __init__(self, args):
        QtGui.QWidget.__init__(self, args)
        
        editorLayout = QtGui.QVBoxLayout(self)
        
        self.splitter1 = QtGui.QSplitter(self)
        self.splitter1.setOrientation(QtCore.Qt.Horizontal)
        self.splitter1.setContentsMargins(0,0,0,0)
        self.mainEditor = toolView.ToolBook(self.splitter1)

        self.toolBox = QtGui.QStackedWidget(self.splitter1)

        editorLayout.addWidget(self.splitter1)
        
        self.splitter1.setStretchFactor(0, 2)
        self.splitter1.setStretchFactor(1, 1)
        
        self.viewMode = "viewer"
        
        self.editorList = {}
        
        self._connectSlots()
        
    def _connectSlots(self):
        QtCore.QObject.connect( self.mainEditor, QtCore.SIGNAL("editorSelected"), self.SelectEditor)
        QtCore.QObject.connect( self.mainEditor, QtCore.SIGNAL("editorClose"), self.CloseEditor)
        
    def count(self):
        return self.mainEditor.count()
        
    def ActiveWidgetHasChanges(self):
        return self.mainEditor.currentWidget().HasChanges()
        
    def SelectEditor(self, theEditor):
        self.toolBox.setCurrentWidget( self.editorList[theEditor.idstr()]['toolbox'] )
        self.parent().SetEditorActions( self.editorList[theEditor.idstr()]['actions'] )
        self.mainEditor.currentWidget().FocusViewer()
        
        self.emit( QtCore.SIGNAL("editorSelected") )

    def CloseEditor(self, theEditor):
        if theEditor.HasChanges():
            theResource = theEditor.GetResource()
            theFilename = "<unnamed>"
            if theResource is not None:
                theFilename = theResource.GetName()
            
            userWants = QtGui.QMessageBox.warning(self, 
                                    theFilename + " has changes",
                                    theFilename + " has changes that have not been saved.  Do you want to save them now?",
                                    QtGui.QMessageBox.Yes | QtGui.QMessageBox.No | QtGui.QMessageBox.Cancel,
                                    QtGui.QMessageBox.Yes )
            if userWants == QtGui.QMessageBox.Yes:
                thePath = ""
                if theFilename == "<unnamed>":
                    thePath = GetSaveFile()
                else:
                    thePath = self.editorList[theEditor.idstr()]['path']
                self.SaveResource(theEditor, thePath)
            elif userWants == QtGui.QMessageBox.No:
                pass
            else:
                return
        self.toolBox.removeWidget( self.editorList[theEditor.idstr()]['toolbox'] )
        self.parent().RemoveEditorActions()
        self.mainEditor.removeTab( self.mainEditor.indexOf(theEditor) )

    def NewEditor(self, editorType):
        if editorType is not None:
            anEditor = editorType(self, self.toolBox)
        else:
            anEditor = genericEditor.genericEditor(self, None)
        anEditor.NewResource()
        
        theStack = EditorStack(self, self.viewMode)
        if editorType is not None:
            theStack.AddViewWidget(anEditor)
            theStack.AddSourceWidget(genericEditor.genericEditor(self, None) )
        else:
            theStack.AddSourceWidget(anEditor)
        theStack.FocusViewer(self.viewMode)
        
        self.mainEditor.addTab(theStack, "<unnamed>" )
        
        theToolBox = anEditor.GetToolBox()
        self.toolBox.addWidget(theToolBox)
        
        theActionList = anEditor.GetActions()
        
        self.editorList[anEditor.idstr()] = { 'editor' : anEditor,
                                                    'widgetstack' : theStack,
                                                    'toolbox' : theToolBox,
                                                    'actions' : theActionList,
                                                    'resource' : anEditor.GetResource(),
                                                    'path' : None }
        
        self.mainEditor.setCurrentWidget(anEditor)

        QtCore.QObject.connect( anEditor, QtCore.SIGNAL("mouseHover"), self.mouseHover)
        QtCore.QObject.connect( anEditor, QtCore.SIGNAL("modificationChanged"), self.ContentsChanged)
        self.emit( QtCore.SIGNAL("editorSelected") )

        return theStack

    def HasAnyChanges(self):
        for a in range(self.mainEditor.count() ):
            if self.mainEditor.widget(a).HasChanges():
                return True
        return False

    def SaveAll(self):
        theActiveWidget = self.mainEditor.currentWidget()
        for a in range(self.mainEditor.count() ):
            self.SaveResource( self.mainEditor.widget(a), self.editorList[anEditor.idstr()]['path'] )
        self.mainEditor.setCurrentWidget(theActiveWidget)

    def SaveResource(self, anEditor, thePath):
        for a in anEditor.CommitToDom():
            print a
        theResource = anEditor.GetResource()
        if thePath is None:
            self.mainEditor.setCurrentWidget(anEditor)
            thePath = GetSaveFile(self)
        if theResource is not None:
            theResource.SaveFile(thePath)

    def SaveActiveResource(self, thePath=None):
        anEditor = self.mainEditor.currentWidget()
        if thePath is None:
            thePath = self.editorList[anEditor.idstr()]['path']
            print thePath
        else:
            self.editorList[anEditor.idstr()]['path'] = thePath
        self.SaveResource(anEditor, thePath)

    def LoadResource(self, theResourcePath):
        theResource = resource_parser.LoadResource(theResourcePath)
        if theResource.GetType() == "aamap":
            anEditor = self.NewEditor(mapEditor.mapView)
        else:
            anEditor = self.NewEditor(None)
        anEditor.LoadResource(theResource)
        
        self.editorList[anEditor.idstr()]['path'] = theResourcePath
        
        self.mainEditor.setTabText(self.mainEditor.indexOf(self.editorList[anEditor.idstr()]['widgetstack']), theResource.GetName() )
        self.emit( QtCore.SIGNAL("activeContentChanged") )
        
    def ContentsChanged(self, state):
        self.emit( QtCore.SIGNAL("modificationChanged"), state)
        
    def SetViewSource(self, state):
        if state is True:
            self.viewMode = "source"
        else:
            self.viewMode = "viewer"

        self.mainEditor.currentWidget().FocusViewer(self.viewMode)
        
    def mouseHover(self, eve):
        self.emit(QtCore.SIGNAL("mouseHover"), eve)

